@@ -9,10 +9,16 @@
 #define _NGINX_H_INCLUDED_
 
 
-#define nginx_version      1006003
-#define NGINX_VERSION      "1.6.3"
+#define nginx_version      1007007
+#define NGINX_VERSION      "1.7.7"
 #define NGINX_VER          "nginx/" NGINX_VERSION
 
+#ifdef NGX_BUILD
+#define NGINX_VER_BUILD    NGINX_VER " (" NGX_BUILD ")"
+#else
+#define NGINX_VER_BUILD    NGINX_VER
+#endif
+
 #define NGINX_VAR          "NGINX"
 #define NGX_OLDPID_EXT     ".oldbin"
 
