@@ -248,7 +248,7 @@ char *ngx_log_set_log(ngx_conf_t *cf, ngx_log_t **head);
 static ngx_inline void
 ngx_write_stderr(char *text)
 {
-    (void) ngx_write_fd(ngx_stderr, text, strlen(text));
+    (void) ngx_write_fd(ngx_stderr, text, ngx_strlen(text));
 }
 
 
