@@ -1568,6 +1568,8 @@ ngx_resolver_process_a(ngx_resolver_t *r, u_char *buf, size_t last,
 
         ngx_rbtree_delete(&r->name_rbtree, &rn->node);
 
+        ngx_resolver_free_node(r, rn);
+
         /* unlock name mutex */
 
         while (next) {
@@ -1578,8 +1580,6 @@ ngx_resolver_process_a(ngx_resolver_t *r, u_char *buf, size_t last,
             ctx->handler(ctx);
         }
 
-        ngx_resolver_free_node(r, rn);
-
         return;
     }
 
@@ -2143,6 +2143,8 @@ valid:
 
         ngx_rbtree_delete(tree, &rn->node);
 
+        ngx_resolver_free_node(r, rn);
+
         /* unlock addr mutex */
 
         while (next) {
@@ -2153,8 +2155,6 @@ valid:
             ctx->handler(ctx);
         }
 
-        ngx_resolver_free_node(r, rn);
-
         return;
     }
 
@@ -2747,8 +2747,7 @@ done:
     }
 
     if (len == -1) {
-        name->len = 0;
-        name->data = NULL;
+        ngx_str_null(name);
         return NGX_OK;
     }
 
@@ -3083,17 +3082,6 @@ ngx_udp_connect(ngx_udp_connection_t *uc)
 
     c->number = ngx_atomic_fetch_add(ngx_connection_counter, 1);
 
-#if (NGX_THREADS)
-
-    /* TODO: lock event when call completion handler */
-
-    rev->lock = &c->lock;
-    wev->lock = &c->lock;
-    rev->own_lock = &c->lock;
-    wev->own_lock = &c->lock;
-
-#endif
-
     ngx_log_debug3(NGX_LOG_DEBUG_EVENT, &uc->log, 0,
                    "connect to %V, fd:%d #%uA", &uc->server, s, c->number);
 
